// I2S - 4 Wire, 2 x 16 Bit, 48 KHz

#include <stdio.h>
#include "pico/stdlib.h"
#include "hardware/pio.h"

#include "MCLK.pio.h"														//pioasm-files
#include "LRCLK.pio.h"
#include "BCLK.pio.h"
#include "DATA.pio.h"

#define PIO0_CTRL ((volatile uint32_t*) 0x50200000)							//Adresse des PIO0-Control-Registers

int main() {

set_sys_clock_pll(1548000000,7,3);											//Systemclock auf 73.714 MHz

pio_gpio_init(pio0, 12);													//GPIO 12 = MCLK 
pio_gpio_init(pio0, 13);													//GPIO 13 = LRCLK
pio_gpio_init(pio0, 14);													//GPIO 14 = BCLK
pio_gpio_init(pio0, 15);													//GPIO 15 = DATA

uint offsetMCLK = pio_add_program(pio0, &MCLK_program);    					//Offset Startadresse MCLK
uint offsetLRCLK = pio_add_program(pio0, &LRCLK_program);    		 		//Offset Startadresse LRCLK
uint offsetBCLK = pio_add_program(pio0, &BCLK_program);    		 			//Offset Startadresse BCLK
uint offsetDATA = pio_add_program(pio0, &DATA_program);    		 			//Offset Startadresse DATA

float MCLKdiv = 3;															//gültiger Bereich: 1 <= MCLKdiv <= 255

//init MCLK
pio_sm_set_consecutive_pindirs(pio0, 0, 12, 1, true);						//(pio 0, StateMachine 0, GPIO 12, 1 Pin, true=OUT)
pio_sm_config cMCLK = MCLK_program_get_default_config(offsetMCLK);			//Standartkonfiguration für StateMachine 0 übernehmen
sm_config_set_set_pins(&cMCLK, 12, 1);										//set pin = GPIO 12, 1 Pin
sm_config_set_clkdiv(&cMCLK,MCLKdiv);										//Clockdivider für StatMachine 0 setzen 
pio_sm_init(pio0, 0, offsetMCLK, &cMCLK);									//(pio 0, StateMachine 0, Startadresse, Konfiguration)

//init LRCLK
pio_sm_set_consecutive_pindirs(pio0, 1, 13, 1, true);						//(pio 0, StateMachine 1, GPIO 13, 1 Pin, true=OUT)
pio_sm_config cLRCLK = LRCLK_program_get_default_config(offsetLRCLK);		//Standartkonfiguration für StateMachine 1 übernehmen
sm_config_set_set_pins(&cLRCLK, 13, 1);										//set pin = GPIO 13, 1 Pin
sm_config_set_clkdiv(&cLRCLK,(MCLKdiv * 256));								//Clockdivider für StatMachine 1 setzen 
pio_sm_init(pio0, 1, offsetLRCLK, &cLRCLK);									//(pio 0, StateMachine 1, Startadresse, Konfiguration)
		
//init BCLK
pio_sm_set_consecutive_pindirs(pio0, 2, 14, 1, true);						//(pio 0, StateMachine 2, GPIO 14, 1 Pin, true=OUT)
pio_sm_config cBCLK = BCLK_program_get_default_config(offsetBCLK);			//Standartkonfiguration für StateMachine 2 übernehmen
sm_config_set_set_pins(&cBCLK, 14, 1);										//set pin = GPIO 14, 1 Pin
sm_config_set_clkdiv(&cBCLK,(MCLKdiv * 8));									//Clockdivider für StatMachine 2 setzen 
pio_sm_init(pio0, 2, offsetBCLK, &cBCLK);									//(pio 0, StateMachine 2, Startadresse, Konfiguration)

//init DATA
pio_sm_set_consecutive_pindirs(pio0, 3, 15, 1, true);						//(pio 0, StateMachine 3, GPIO 15, 1 Pin, true=OUT)
pio_sm_config cDATA = DATA_program_get_default_config(offsetDATA);			//Standartkonfiguration für StateMachine 2 übernehmen
sm_config_set_out_pins(&cDATA, 15, 1);										//out pin = GPIO 15, 1 Pin
sm_config_set_clkdiv(&cDATA,(MCLKdiv * 4));									//Clockdivider für StatMachine 3 setzen 
sm_config_set_out_shift(&cDATA, 0, 1, 32);									// FIFO => FILO , autopull enabled, 32 Bit
pio_sm_init(pio0, 3, offsetDATA, &cDATA);									//(pio 0, StateMachine 3, Startadresse, Konfiguration)

pio_sm_put(pio0,3,0xa000a000);												//beschreibe TX-FIFO Statemachine 3

*PIO0_CTRL = 0x00000fff;													//pio 0, Statemachines 0, 1, 2 und 3 synchron starten

 while (true) {
 pio_sm_put(pio0,3,0xa000a000);												//beschreibe TX-FIFO Statemachine 3
 }
 
}
